/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
package org.jdtaus.core.text.ri.combined;

import java.util.Locale;
import javax.swing.event.EventListenerList;
import org.jdtaus.core.container.ContainerFactory;
import org.jdtaus.core.container.ContextFactory;
import org.jdtaus.core.container.ContextInitializer;
import org.jdtaus.core.container.Dependency;
import org.jdtaus.core.container.Implementation;
import org.jdtaus.core.container.ModelFactory;
import org.jdtaus.core.container.Properties;
import org.jdtaus.core.container.Property;
import org.jdtaus.core.text.MessageEvent;
import org.jdtaus.core.text.MessageListener;
import org.jdtaus.core.text.spi.ApplicationLogger;
import org.jdtaus.core.logging.spi.Logger;

/**
 * jDTAUS Core SPI {@code ApplicationLogger} reference implementation.
 * <p>The reference implementation will log messages with the logger configured
 * as a dependency to this class using the default locale and then notifies all
 * registered listeners.</p>
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: CombinedApplicationLogger.java 3553 2007-05-25 22:37:09Z schulte2005 $
 *
 * @see org.jdtaus.core.container.Container
 */
public final class CombinedApplicationLogger implements ApplicationLogger
{
    //--Implementation----------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /** Meta-data describing the implementation. */
    private static final Implementation META =
        ModelFactory.getModel().getModules().
        getImplementation(CombinedApplicationLogger.class.getName());

    //----------------------------------------------------------Implementation--
    //--Constructors------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /**
     * Protected <code>CombinedApplicationLogger</code> implementation constructor.
     *
     * @param meta Implementation meta-data.
     *
     * @throws NullPointerException if <code>meta</code> is <code>null</code>.
     */
    protected CombinedApplicationLogger(final Implementation meta)
    {
        super();
        if(meta == null)
        {
            throw new NullPointerException("meta");
        }
        this.initializeProperties(meta.getProperties());
    }
    /**
     * Protected <code>CombinedApplicationLogger</code> dependency constructor.
     *
     * @param meta dependency meta-data.
     *
     * @throws NullPointerException if <code>meta</code> is <code>null</code>.
     */
    protected CombinedApplicationLogger(final Dependency meta)
    {
        super();
        if(meta == null)
        {
            throw new NullPointerException("meta");
        }
        this.initializeProperties(meta.getProperties());
    }

    /**
     * Initializes the properties of the instance.
     *
     * @param meta the property values to initialize the instance with.
     *
     * @throws NullPointerException if {@code meta} is {@code null}.
     */
    protected void initializeProperties(final Properties meta)
    {
        Property p;

        if(meta == null)
        {
            throw new NullPointerException("meta");
        }

    }

    //------------------------------------------------------------Constructors--
    //--Properties--------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.


    //--------------------------------------------------------------Properties--
    //--Dependencies------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /** Configured <code>Logger</code> implementation. */
    private transient Logger _dependency0;

    /**
     * Gets the configured <code>Logger</code> implementation.
     *
     * @return the configured <code>Logger</code> implementation.
     */
    private Logger getLogger()
    {
        Logger ret = null;
        if(this._dependency0 != null)
        {
            ret = this._dependency0;
        }
        else
        {
            ret = (Logger) ContainerFactory.getContainer().
                getDependency(CombinedApplicationLogger.class,
                "Logger");

            if(ModelFactory.getModel().getModules().
                getImplementation(CombinedApplicationLogger.class.getName()).
                getDependencies().getDependency("Logger").
                isBound())
            {
                this._dependency0 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }

    //------------------------------------------------------------Dependencies--
    //--MessageEventSource------------------------------------------------------

    /** Holds {@code MessageListener}s. */
    private EventListenerList listeners = new EventListenerList();

    public void addMessageListener(final MessageListener messageListener)
    {
        if(messageListener == null)
        {
            throw new NullPointerException("messageListener");
        }

        this.listeners.add(MessageListener.class, messageListener);
    }

    public void removeMessageListener(MessageListener messageListener)
    {
        if(messageListener == null)
        {
            throw new NullPointerException("messageListener");
        }

        this.listeners.remove(MessageListener.class, messageListener);
    }

    public MessageListener[] getMessageListeners()
    {
        return (MessageListener[]) this.listeners.
            getListeners(MessageListener.class);

    }

    //------------------------------------------------------MessageEventSource--
    //--ApplicationLogger-------------------------------------------------------

    public void log(final MessageEvent messageEvent)
    {
        if(messageEvent == null)
        {
            throw new NullPointerException("messageEvent");
        }

        final String txt = messageEvent.getMessage().
            getText(Locale.getDefault());

        switch(messageEvent.getType())
        {
            case MessageEvent.INFORMATION:
                this.getLogger().info(txt);
                break;

            case MessageEvent.NOTIFICATION:
            case MessageEvent.WARNING:
                this.getLogger().warn(txt);
                break;

            default:
                this.getLogger().error(txt);

        }

        this.fireOnMessage(messageEvent);
    }

    //-------------------------------------------------------ApplicationLogger--
    //--CombinedApplicationLogger-----------------------------------------------

    /** Creates a new {@code CombinedApplicationLogger} instance. */
    public CombinedApplicationLogger()
    {
        this(CombinedApplicationLogger.META);
    }

    /**
     * Notifies all registered {@code MessageListener}s.
     *
     * @param e the event to provide to all listeners.
     *
     * @throws NullPointerException if {@code e} is {@code null}.
     */
    protected void fireOnMessage(final MessageEvent e)
    {
        if(e == null)
        {
            throw new NullPointerException("e");
        }

        final Object[] listeners = this.listeners.getListenerList();
        for (int i = listeners.length - 2; i >= 0; i -= 2)
        {
            if (listeners[i] == MessageListener.class)
            {
                ((MessageListener) listeners[i + 1]).onMessage(e);
            }
        }
    }

    //-----------------------------------------------CombinedApplicationLogger--
}
